# -*- coding: utf-8 -*-
#
# Version 1.1
"""
Module avec les fonctions de la classe de Seconde 2018 pour le lycée diderot (marseille).
On prend comme fichier de départ le module de l'irem d'Amiens
http://download.tuxfamily.org/amienspython/lycee.py
Licence http://www.cecill.info/

Il y a des fonctions d'Amiens qui devaient être renommées / améliorées.
On nomme les fonctions en FRANCAIS.

Le module lycee est un module python réalisé par le groupe AMIENS PYTHON
a pour objectif de simplifier un certain nombre de manipulations
avec python au lycée (cosinus en degré, calcul d'une moyenne d'une liste,
représentation statistiques variées, ...)

Pour l'utiliser, il suffit d'ajouter en début de programme

from lycee import *
"""


import math

from arithmetique import *
from trigo import *
from stats_proba import *
from fonctions_usuelles import *
from vecteurs import *
from chaines import *
from listes import *

pi = math.pi
